==> MiningMart-1.0-src.zip: MiningMart-1.0/src/edu/udo/cs/miningmart/gui/stepsettings/ListFeatureEditor.java <==
/*
 * MiningMart Version 1.0
 * 
 * Copyright (C) 2006 Martin Scholz, Timm Euler, 
 *                    Daniel Hakenjos, Katharina Morik
 *
 * Contact: miningmart@ls8.cs.uni-dortmund.de
 *
 * A list of contributing developers (other than the copyright 
 * holders) can be found at
 * http://mmart.cs.uni-dortmund.de/downloads/download.html
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program, see the file MM_HOME/LICENSE; if not, write
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301, USA.
 */
package edu.udo.cs.miningmart.gui.stepsettings;

import java.awt.event.ActionEvent;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Vector;

import org.musoft.limo.application.Resource;

import edu.udo.cs.miningmart.exception.M4Exception;
import edu.udo.cs.miningmart.gui.application.MiningMartApplication;
import edu.udo.cs.miningmart.m4.Concept;
import edu.udo.cs.miningmart.m4.Feature;
import edu.udo.cs.miningmart.m4.M4Interface;
import edu.udo.cs.miningmart.m4.OpParam;
import edu.udo.cs.miningmart.m4.utils.Print;

/**
 *An editor and renderer for multi-selection of <code>Feature</code>s.
 * @author Daniel Hakenjos
 * @version $Id: ListFeatureEditor.java,v 1.5 2006/05/23 09:14:30 euler Exp $
 */
public class ListFeatureEditor extends ListSelectEditor {
	
	private StepParameterTable table;
	
	private Vector features;
	
        /**
         *Construct a new <code>ListFeatureEditor</code>.
         */
	public ListFeatureEditor(MiningMartApplication app,StepParameterTable table, OpParam opParam){
		super(app,table.getTheStep(),opParam);
		this.table=table;

	}
	
	public void init(){
		super.init();
		
		try{
			this.selected_data=new String[0];

			Collection coll=getFeatureCollection();
			if (coll!=null){
				this.selected_data=new String[coll.size()];
				Iterator iter=coll.iterator();
				int index=0;
				while (iter.hasNext()){
//					if (opParam.isInput()){
						Feature feat=(Feature) iter.next();
						String name=new String("");
						if (feat.getConcept()!=null){
							name=feat.getConcept().getName();
						}
						selected_data[index]=(feat).getName()+" ["+name+"]";
/*					}else{
						selected_data[index]=(String) iter.next();
					}
*/					index++;
				}
			}
			setDataItems(selected_data);
		}catch(M4Exception error){
			M4Interface.print.doPrint(Print.ERROR,error.getMessage(),error);
		}
	}
	
	

	/**
	 * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		try{
			Collection conceptparams=step.getPossibleConceptsForParam(opParam);
			this.data=new String[conceptparams.size()];

			Iterator iter=conceptparams.iterator();
			int d=0;
			while (iter.hasNext()){
				data[d]=(String) iter.next();
				d++;
			}

			Collection concepts=new LinkedList();			
			for (int i=0;i<data.length;i++){
				concepts.addAll(table.getConceptEditorValues(data[i]));
			}


			Collection battribs=new LinkedList();
			iter=concepts.iterator();
			Concept concept;
			while (iter.hasNext()){
				concept=M4Interface.getCurrentCase().getConcept((String) iter.next());
				if (concept!=null){
					Collection c=concept.getAllBaseAttributes();
					c.addAll(concept.getAllMultiColumnFeatures());
					Iterator iter2=c.iterator();
					Feature feat;					
					while (iter2.hasNext()){
						feat=(Feature) iter2.next();
						if (step.isVisible(feat)){
							battribs.add(feat);
						}
					}
				}
			}

			this.data=new String[battribs.size()];
			
			iter=battribs.iterator();
			int c=0;
			Feature feat;
			String name;
			while (iter.hasNext()){
				feat=(Feature) iter.next();
				name=new String("");
				if (feat.getConcept()!=null){
					name=feat.getConcept().getName();
				}
				data[c]=feat.getName()+" ["+name+"]";
				c++;
			}
	
			ListSelectDialog dialog=new ListSelectDialog(app,Resource.getString("STEPSETTINGS_TITLE_FEATURE"),Resource.getString("STEPSETTINGS_TITLE_SELECT_FEATURE"),data,selected_data,true,opParam.getMinArg(),opParam.getMaxArg());
			if (dialog.getExitAction()==ListSelectDialog.OK){
				selected_data=dialog.getSelectedDataItems();
				this.setDataItems(selected_data);

				LinkedList list=new LinkedList();
				for(int s=0;s<selected_data.length;s++){
					iter=battribs.iterator();
					Feature selectedF=null;
					boolean found=false;
					while ((iter.hasNext())&&(!found)){
						selectedF=(Feature) iter.next();
						name=new String("");
						if (selectedF.getConcept()!=null){
							name=selectedF.getConcept().getName();
						}
						// if ((selectedF.getName()+" ["+name+"]").equals(selected_data[s])){
						String nameOfSelected = selected_data[s];
						if (nameOfSelected.indexOf("[") > -1) {
							nameOfSelected = selected_data[s].substring(0, selected_data[s].indexOf("[")).trim();
						}
						if (nameOfSelected.equalsIgnoreCase(selectedF.getName())) {
							found=true;
						}
					}
					if (found){
						list.add(selectedF);
					}
				}
				setParameterCollection(list);
				this.fireStateChanged();

			}
		}catch(M4Exception error){
			M4Interface.print.doPrint(Print.ERROR,error.getMessage(),error);
		}
	}

	private void initFeatures(){
		features=new Vector();
		numberofloops=step.getLoopCount();
		if (numberofloops==0){
			numberofloops=1;
		}

		try{
			Collection coll;
			for(int i=0;i<numberofloops;i++){
				if (opParam.isCoordinated()){
					//coordinierte Parameter so auslesen
					coll=step.getParameter(opParam,0);					
					if (coll==null)	coll=new LinkedList();
					features.add(coll);
				}else if ((opParam.isLoopable())&&(numberofloops==1)){
					//opParam ist loopable, mechanismus wird im step nicht genutzt
					coll=step.getParameter(opParam,0);					
					if (coll==null)	coll=new LinkedList();
					features.add(coll);
					
				}else if ((opParam.isLoopable())&&(numberofloops>1)){
					//opParam ist loopable, mechanismus wird im step genutzt
					//also auslesen von 1...numberofloops
					coll=step.getParameter(opParam,i+1);					
					if (coll==null)	coll=new LinkedList();
					features.add(coll);
					
				}else{
					coll=step.getParameter(opParam,0);					
					if (coll==null)	coll=new LinkedList();
					features.add(coll);
				}
			}
		}catch(M4Exception error){
			M4Interface.print.doPrint(Print.ERROR,error.getMessage(),error);
		}
	}

	/* (non-Javadoc)
	 * @see edu.udo.cs.miningmart.gui.stepsettings.ParameterEditor#setParameterCollection(java.util.Collection, int)
	 */
	public void setParameterCollection(Collection collection, int loopnumber) {
		super.setParameterCollection(collection, loopnumber);
		boolean loopable=true;
		try{
			loopable=opParam.isLoopable();
		}catch(M4Exception error){
			M4Interface.print.doPrint(Print.ERROR,error.getMessage(),error);
			loopable=true;
		}

		if (features==null){
			initFeatures();
		}

		if (loopable){
			features.set(loopnumber-1, collection);
		}else{
			for(int i=0;i<paramcollections.size();i++){
				features.set(i, collection);
			}
		}
	}

	/**
	 * Gets the <code>Collection</code> of the Parameter for the specified loopnumber.
	 */
	private Collection getFeatureCollection(int loopnumber){
		boolean loopable=true;
		try{
			loopable=opParam.isLoopable();
		}catch(M4Exception error){
			M4Interface.print.doPrint(Print.ERROR,error.getMessage(),error);
			loopable=true;
		}
		if (features==null){
			initFeatures();
		}
		if (loopable){		
			return (Collection) features.get(loopnumber-1);
		}else{
			return (Collection) features.get(0);
		}
	}
	
	/**
	 * Gets the <code>Collection</code> for the current loopnumber.
	 */
	private Collection getFeatureCollection(){
		return getFeatureCollection(loopnumber);
	}
	
	/* (non-Javadoc)
	 * @see edu.udo.cs.miningmart.gui.stepsettings.ParameterEditor#makeOutput(boolean, boolean, boolean, boolean)
	 */
	public void makeOutput(boolean inputchanged,boolean outputchanged,boolean loopcountlower,	boolean loopcounthigher)	throws M4Exception {
		super.makeOutput(inputchanged,outputchanged,loopcountlower,loopcounthigher);
		initFeatures();
	}

	/* (non-Javadoc)
	 * @see edu.udo.cs.miningmart.gui.stepsettings.ParameterEditor#existsOutput()
	 */
	public boolean existsOutput() {
		if (opParam.isInput())
			return false;
	
		Collection coll;
		Collection actual;
		for(int loop=1;loop<=numberofloops;loop++){
			coll=(Collection) this.oldparamcollections.get(loop-1);
			actual=(Collection) this.paramcollections.get(loop-1);
			if (coll==null){
				return false;
			}
			if (coll.isEmpty()){
				return false;
			}
			if (actual==null){
				actual=new LinkedList();
			}
			if (coll.size()!=actual.size()){
				return false;
			}else{
				//pr\x{00FC}fe, ob die Namen passen
				Iterator iter=coll.iterator();
				while (iter.hasNext()){
					if (!actual.contains(iter.next())){
						return false;
					}
				}
			}
		}
			
		//gibt nur true zurueck, wenn in allen loops die gleiche nicht leere Anzahl von Eintraegen vorhanden ist und die Eintraege auch 
		//noch im Namen uebereinstimmen
		return true;
	}

	/* (non-Javadoc)
	 * @see edu.udo.cs.miningmart.gui.stepsettings.ParameterEditor#setNumberOfLoops(int)
	 */
	public void setNumberOfLoops(int numberofloops) {
		if (numberofloops==0){
			numberofloops=1;
		}
	
		if (features==null){
			initFeatures();
		}
		if (features.size()<numberofloops){
			while (features.size()!=numberofloops){
				features.add(new LinkedList());
			}
		}else if (features.size()>numberofloops){
			while (features.size()!=numberofloops){
				features.remove(paramcollections.size()-1);
			}
		}
	
		super.setNumberOfLoops(numberofloops);
	}

}
/*
$Log: ListFeatureEditor.java,v $
Revision 1.5  2006/05/23 09:14:30  euler
Bugfix in 'actionPerformed'.

Revision 1.4  2006/04/11 14:10:13  euler
Updated license text.

Revision 1.3  2006/04/06 16:31:13  euler
Prepended license remark.

Revision 1.2  2006/02/02 15:38:46  euler
Bugfix

Revision 1.1  2006/01/03 09:54:14  hakenjos
Initial version!

*/

==> MiningMart-1.1-src.zip: MiningMart-1.1/src/edu/udo/cs/miningmart/gui/stepsettings/ListFeatureEditor.java <==
/*
 * MiningMart Version 1.1
 * 
 * Copyright (C) 2006 Martin Scholz, Timm Euler, 
 *                    Daniel Hakenjos, Katharina Morik
 *
 * Contact: miningmart@ls8.cs.uni-dortmund.de
 *
 * A list of contributing developers (other than the copyright 
 * holders) can be found at
 * http://mmart.cs.uni-dortmund.de/downloads/download.html
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program, see the file MM_HOME/LICENSE; if not, write
 * to the Free Software Foundation, Inc., 51 Franklin Street, Fifth
 * Floor, Boston, MA 02110-1301, USA.
 */
package edu.udo.cs.miningmart.gui.stepsettings;

import java.awt.event.ActionEvent;
import java.util.Collection;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Vector;

import org.musoft.limo.application.Resource;

import edu.udo.cs.miningmart.exception.M4Exception;
import edu.udo.cs.miningmart.gui.application.MiningMartApplication;
import edu.udo.cs.miningmart.m4.Feature;
import edu.udo.cs.miningmart.m4.M4Interface;
import edu.udo.cs.miningmart.m4.OpParam;
import edu.udo.cs.miningmart.m4.utils.Print;

/**
 *An editor and renderer for multi-selection of <code>Feature</code>s.
 * @author Daniel Hakenjos
 * @version $Id: ListFeatureEditor.java,v 1.12 2006/09/27 14:59:59 euler Exp $
 */
public class ListFeatureEditor extends ListSelectEditor {
	
	private Vector features;
	
        /**
         *Construct a new <code>ListFeatureEditor</code>.
         */
	public ListFeatureEditor(MiningMartApplication app,StepParameterTable table, OpParam opParam){
		super(app,table.getTheStep(),opParam);
		this.table=table;

	}
	
	public void init(){
		super.init();
		
		try{
			this.selected_data=new String[0];

			Collection coll=getFeatureCollection();
			if (coll!=null){
				this.selected_data=new String[coll.size()];
				Iterator iter=coll.iterator();
				int index=0;
				while (iter.hasNext()){
//					if (opParam.isInput()){
						Feature feat=(Feature) iter.next();
						String name=new String("");
						if (feat.getConcept()!=null){
							name=feat.getConcept().getName();
						}
						selected_data[index]=(feat).getName()+" ["+name+"]";
/*					}else{
						selected_data[index]=(String) iter.next();
					}
*/					index++;
				}
			}
			setDataItems(selected_data);
		}catch(M4Exception error){
			M4Interface.print.doPrint(Print.ERROR,error.getMessage(),error);
		}
	}
	
	

	/**
	 * @see java.awt.event.ActionListener#actionPerformed(ActionEvent)
	 */
	public void actionPerformed(ActionEvent e) {
		try{

			Collection battribs = this.getListOfAttributesToSelectAttributeFrom(opParam);
			if (battribs == null) {
				this.showErrorMessage();
				return;
			}

			this.data = new String[battribs.size()];

			Iterator iter = battribs.iterator();
			int c=0;
			Feature feat;
			String name;
			while (iter.hasNext()){
				feat=(Feature) iter.next();
				name=new String("");
				if (feat.getConcept()!=null){
					name=feat.getConcept().getName();
				}
				data[c]=feat.getName()+" ["+name+"]";
				c++;
			}
	
			ListSelectDialog dialog=new ListSelectDialog(app,Resource.getString("STEPSETTINGS_TITLE_FEATURE"),Resource.getString("STEPSETTINGS_TITLE_SELECT_FEATURE"),data,selected_data,true,opParam.getMinArg(),opParam.getMaxArg());
			if (dialog.getExitAction()==ListSelectDialog.OK){
				selected_data=dialog.getSelectedDataItems();
				this.setDataItems(selected_data);

				LinkedList list=new LinkedList();
				for(int s=0;s<selected_data.length;s++){
					iter=battribs.iterator();
					Feature selectedF=null;
					boolean found=false;
					while ((iter.hasNext())&&(!found)){
						selectedF=(Feature) iter.next();
						String conceptName = "";
						if (selectedF.getConcept()!=null){
							conceptName=selectedF.getConcept().getName();
						}
						// if ((selectedF.getName()+" ["+name+"]").equals(selected_data[s])){
						String nameOfCandidate = selected_data[s];
						String nameOfSelectedFeatureWithConcept = selectedF.getName();
						// if there is only one input concept, only the names
						// of the attribs have to match; if there are more input concepts,
						// both the names of the attribs and their concepts have to match:
						if (this.step.getAllInputConcepts().size() == 1) {
							int additionIndex = nameOfCandidate.indexOf("[");
							if (additionIndex > -1) {
								nameOfCandidate = nameOfCandidate.substring(0, additionIndex).trim();
							}
						}
						else {
							if (nameOfCandidate.indexOf("[") > -1) {
								nameOfSelectedFeatureWithConcept += " [" + conceptName + "]";
							}
						}
						if (nameOfCandidate.equalsIgnoreCase(nameOfSelectedFeatureWithConcept)) {
							found=true;
						}
					}
					if (found){
						list.add(selectedF);
					}
				}
				setParameterCollection(list);
				this.fireStateChanged();

			}
		}catch(M4Exception error){
			M4Interface.print.doPrint(Print.ERROR,error.getMessage(),error);
		}
	}

	private void initFeatures(){
		features=new Vector();
		numberofloops=step.getLoopCount();
		if (numberofloops==0){
			numberofloops=1;
		}

		try{
			Collection coll;
			for(int i=0;i<numberofloops;i++){
				if (opParam.isCoordinated()){
					//coordinierte Parameter so auslesen
					coll=step.getParameter(opParam,0);					
					if (coll==null)	coll=new LinkedList();
					features.add(coll);
				}else if ((opParam.isLoopable())&&(numberofloops==1)){
					//opParam ist loopable, mechanismus wird im step nicht genutzt
					coll=step.getParameter(opParam,0);					
					if (coll==null)	coll=new LinkedList();
					features.add(coll);
					
				}else if ((opParam.isLoopable())&&(numberofloops>1)){
					//opParam ist loopable, mechanismus wird im step genutzt
					//also auslesen von 1...numberofloops
					coll=step.getParameter(opParam,i+1);					
					if (coll==null)	coll=new LinkedList();
					features.add(coll);
					
				}else{
					coll=step.getParameter(opParam,0);					
					if (coll==null)	coll=new LinkedList();
					features.add(coll);
				}
			}
		}catch(M4Exception error){
			M4Interface.print.doPrint(Print.ERROR,error.getMessage(),error);
		}
	}

	/* (non-Javadoc)
	 * @see edu.udo.cs.miningmart.gui.stepsettings.ParameterEditor#setParameterCollection(java.util.Collection, int)
	 */
	public void setParameterCollection(Collection collection, int loopnumber) {
		super.setParameterCollection(collection, loopnumber);
		boolean loopable=true;
		try{
			loopable=opParam.isLoopable();
		}catch(M4Exception error){
			M4Interface.print.doPrint(Print.ERROR,error.getMessage(),error);
			loopable=true;
		}

		if (features==null){
			initFeatures();
		}

		if (loopable){
			features.set(loopnumber-1, collection);
		}else{
			for(int i=0;i<paramcollections.size();i++){
				features.set(i, collection);
			}
		}
	}

	/**
	 * Gets the <code>Collection</code> of the Parameter for the specified loopnumber.
	 */
	private Collection getFeatureCollection(int loopnumber){
		boolean loopable=true;
		try{
			loopable=opParam.isLoopable();
		}catch(M4Exception error){
			M4Interface.print.doPrint(Print.ERROR,error.getMessage(),error);
			loopable=true;
		}
		if (features==null){
			initFeatures();
		}
		if (loopable){		
			return (Collection) features.get(loopnumber-1);
		}else{
			return (Collection) features.get(0);
		}
	}
	
	/**
	 * Gets the <code>Collection</code> for the current loopnumber.
	 */
	private Collection getFeatureCollection(){
		return getFeatureCollection(loopnumber);
	}
	
	/* (non-Javadoc)
	 * @see edu.udo.cs.miningmart.gui.stepsettings.ParameterEditor#makeOutput(boolean, boolean, boolean, boolean)
	 */
	public void makeOutput(boolean inputchanged,boolean outputchanged,boolean loopcountlower,	boolean loopcounthigher)	throws M4Exception {
		super.makeOutput(inputchanged,outputchanged,loopcountlower,loopcounthigher);
		initFeatures();
	}

	/* (non-Javadoc)
	 * @see edu.udo.cs.miningmart.gui.stepsettings.ParameterEditor#existsOutput()
	 */
	public boolean existsOutput() {
		if (opParam.isInput())
			return false;
	
		Collection coll;
		Collection actual;
		for(int loop=1;loop<=numberofloops;loop++){
			coll=(Collection) this.oldparamcollections.get(loop-1);
			actual=(Collection) this.paramcollections.get(loop-1);
			if (coll==null){
				return false;
			}
			if (coll.isEmpty()){
				return false;
			}
			if (actual==null){
				actual=new LinkedList();
			}
			if (coll.size()!=actual.size()){
				return false;
			}else{
				//pr\x{00FC}fe, ob die Namen passen
				Iterator iter=coll.iterator();
				while (iter.hasNext()){
					if (!actual.contains(iter.next())){
						return false;
					}
				}
			}
		}
			
		//gibt nur true zurueck, wenn in allen loops die gleiche nicht leere Anzahl von Eintraegen vorhanden ist und die Eintraege auch 
		//noch im Namen uebereinstimmen
		return true;
	}

	/* (non-Javadoc)
	 * @see edu.udo.cs.miningmart.gui.stepsettings.ParameterEditor#setNumberOfLoops(int)
	 */
	public void setNumberOfLoops(int numberofloops) {
		if (numberofloops==0){
			numberofloops=1;
		}
	
		if (features==null){
			initFeatures();
		}
		if (features.size()<numberofloops){
			while (features.size()!=numberofloops){
				features.add(new LinkedList());
			}
		}else if (features.size()>numberofloops){
			int i = features.size();
			while (i!=numberofloops){
				features.remove(i-1);
				i--;
			}
		}
	
		super.setNumberOfLoops(numberofloops);
	}

}
/*
$Log: ListFeatureEditor.java,v $
Revision 1.12  2006/09/27 14:59:59  euler
New version 1.1

Revision 1.11  2006/09/11 14:46:38  euler
*** empty log message ***

Revision 1.10  2006/08/20 16:55:07  euler
bugs fixed

Revision 1.9  2006/06/20 07:51:41  euler
Bugfixes

Revision 1.8  2006/06/19 15:37:13  euler
Bugfixes

Revision 1.7  2006/06/18 15:13:05  euler
Bugfixes

Revision 1.6  2006/06/16 16:26:32  euler
New operator "FeatureConstructionByRelation"

Revision 1.5  2006/05/23 09:14:30  euler
Bugfix in 'actionPerformed'.

Revision 1.4  2006/04/11 14:10:13  euler
Updated license text.

Revision 1.3  2006/04/06 16:31:13  euler
Prepended license remark.

Revision 1.2  2006/02/02 15:38:46  euler
Bugfix

Revision 1.1  2006/01/03 09:54:14  hakenjos
Initial version!

*/
